let doc = """
ncp is a very basic cp written in nim with some modern defaults.

Usage:
  ncp <path>... [--recurse] [--posix] [--verbose] [--interactive]
  ncp (-h | --help)
  ncp [--version]

Options:
  -h, --help            Show this screen.
  -r, --recurse         Enables recursive finding of files (Enabled by default)
  -v, --verbose         Enables verbose output
  -i, --interactive     Prompts the user for action to overwrite
  -p, --posix           Disables default enables
  --version             Show version.
"""


import docopt
import lists
import os
import strformat
import strutils

func exists(check: string): bool =
  return bool(fileExists(check) or dirExists(check))


func stripPath(input: string): string =
  let file = splitPath(input)
  return file.tail

proc isFile(check: string): bool =
  if check.exists:
    return bool(os.getFileInfo(check).kind == os.pcFile)

proc confirm(input: string): bool =
  while true:
    fmt"Overwrite '{input}'? (y/n [n])".echo
    let response = readLine(stdin).toLowerAscii
    if response == "y" or response == "yes":
      return true
    else:
      "Not overwriting".echo
      return false


proc copyFileWithPermissionsLocal(source, dest: string,
    ignorePermissionErrors = true, options = {cfSymlinkFollow}, verbose = false,
    interactive = false) =
  if verbose: fmt"{source} -> {dest}".echo
  copyFile(source, dest, options)
  when not defined(windows):
    try:
      setFilePermissions(dest, getFilePermissions(source), followSymlinks =
        (cfSymlinkFollow in options))
    except:
      if not ignorePermissionErrors:
        raise

proc copyDirWithPermissionsLocal(source, dest: string,
    ignorePermissionErrors = true, verbose = false, interactive = false) =
  if dest.exists and interactive:
    if not confirm(dest): return
  if verbose: fmt"{source} -> {dest}".echo
  createDir(dest)
  when not defined(windows):
    try:
      setFilePermissions(dest, getFilePermissions(source), followSymlinks =
        false)
    except:
      if not ignorePermissionErrors:
        raise
  for kind, path in walkDir(source):
    var noSource = splitPath(path).tail
    if kind == pcDir:
      copyDirWithPermissionsLocal(path, dest / noSource, ignorePermissionErrors,
          verbose = verbose, interactive = interactive)
    else:
      copyFileWithPermissionsLocal(path, dest / noSource,
          ignorePermissionErrors, {cfSymlinkAsIs}, verbose = verbose,
          interactive = interactive)

proc copy(f, to: string, recurse, interactive, verbose: bool) =
  if f.isFile: # input is a file
    if to.exists and not to.isFile or not to.exists: # file to directory
      copyFileWithPermissionsLocal(source = f, dest = to, verbose = verbose)
    else: # file to file does exist
      if interactive and confirm(to):
        if to.isFile:
          to.removeFile
          # Attempts again if file deleted. Disabled verbosity on recursion
          copy(f, to, recurse, interactive, false)
        else: to.removeDir

  else: # f is a directory
    if verbose: fmt"{f} -> {to}".echo
    if recurse:
      copyDirWithPermissionsLocal(f, to, verbose = verbose,
          interactive = interactive)


proc main() =
  var
    from_list = initDoublyLinkedList[string]()
    recurse: bool
    to: string
    interactive: bool
    verbose: bool
    posix: bool

  recurse = true
  interactive = true
  verbose = true

  let args = docopt(doc, version = "cp 0.1")
  let path = args["<path>"]
  if args["--posix"]:
    if not args["--recurse"]: recurse = false
    if not args["--interactive"]: interactive = false
    if not args["--verbose"]: verbose = false

  if path.len <= 1:
    "ncp: Missing file for operand".echo
    "See 'ncp --help' for more info".echo
    quit(QuitFailure)

  # Builds from list. Last option is the output dir
  for i, c in path:
    if i < path.len-1:
      from_list.append(c.join)
    else: to = c.normalizedPath

  for i in from_list:
    copy(i, to, recurse, interactive, verbose)


when isMainModule:
  main()
